#!/usr/bin/perl

use strict;
use warnings "all";

my ($in, $out);
our $oneline = 0;

foreach (@ARGV) {
	if ($_ =~ /-i=([a-z]+)/) 	{$in = $1;}
	if ($_ =~ /-o=([a-z]+)/) 	{$out = $1;}
	if ($_ =~ /-noblock/) 		{$oneline = 1;}
}

my $file = pop(@ARGV);

my $usage = "Universal sequence converter\nconvertseq.pl [OPTIONS] -i=INFORMAT -o=OUTFORMAT FILE\nOptions:\n -noblocks                       Do not block output, write entries to one line if possible \nKnown input formats:\n cw, aln, fasta\nKnown output formats:\n phy, aln, fasta\n\n";

unless (defined($file) && $file =~ /^[^-]/ && defined($in) && defined($out)) {
	print STDERR $usage;
	print STDERR exit;
}

our %hash;
if 	($in eq "aln") 			{%hash = &aln2hash($file);}
elsif 	($in eq "cw") 			{%hash = &cw2hash($file);}
elsif 	($in eq "stk") 			{%hash = &stk2hash($file);}
elsif 	($in eq "fasta") 		{%hash = &fasta2hash($file);}
else 					{print STDERR "Unknown input format\n$usage"; exit;}

if 	($out eq "aln") 		{&hash2aln();}
elsif 	($out eq "fasta") 		{&hash2fasta();}
elsif 	($out eq "phy") 		{&hash2phy();}
else 					{print STDERR "Unknown output format\n$usage"; exit;}



### Output filters
sub hash2fasta {
	my $skip = 80;

	if ($oneline) {
		foreach my $key (keys %hash) {
			print ">$key\n$hash{$key}\n";
		}
	}
	else {
		foreach my $key (keys %hash) {
			my $wrote = 0;			
			print ">$key\n";
			while ($wrote < length($hash{$key})) {
				print substr($hash{$key},$wrote,$skip),"\n";
				$wrote += $skip;
			}
		}
	}
}

sub hash2phy {
	my $length = 0;
	foreach my $key (keys %hash) {
		if ($length < length($hash{$key})) {
			$length = length($hash{$key});
		}
	}

	print scalar(keys %hash)," $length\n";

	my $space = 10;
	my $start = 0;
	my $skip = 60;

	if ($oneline) {
		print STDERR "[Warning] Phy does not support unblocked files, ignoring option!\n";print "\n";
	}
	else {
		while (1) {
			foreach my $key (keys %hash) {
				if ($start == 0) {print &tolength($key,$space);}
				print &tolength(substr($hash{$key},$start),$skip,"-");
				print "\n";
			}
	
			print "\n";

			$start += $skip;
			if ($start + $skip > $length) {
				$skip =	$length-$start;
			}
			if ($skip <= 0) {last;}
			}
	}
}

sub hash2aln {
	print "CLUSTAL\n\n\n";

	my $length;	foreach my $key (keys %hash) {$length = length($hash{$key});	last;}

	my $space = 25;
	my $start = 0;
	my $skip = 30;

	if ($oneline) {
		foreach my $key (keys %hash) {
			print &tolength($key,$space);
			print $hash{$key};
			print "\n";
		}
	}
	else {
		while (1) {
			foreach my $key (keys %hash) {
			print &tolength($key,$space);
			print substr($hash{$key},$start,$skip);
			print "\n";
		}
	
		print &tolength(" ",$space);
		print &empty($skip," ");
		print "\n\n";

		$start += $skip;
		if ($start + $skip > $length) {
			$skip =	$length-$start;
		}
		if ($skip <= 0) {last;}
		}
	}
}


sub empty {
	my $num = shift;
	my $char = shift;
	if (!defined($char)) {$char = "-";}
	my $string = "";
	while ($num-- > 0) {
		$string .= $char;
	}	
	return $string;
}

sub tolength() {
	my $string = shift;
	my $space = shift;
	my $sep = shift;
	if (!defined($sep)) {$sep = " ";}
	if (length($string) >= $space-2) {
		$string = substr($string,0,$space-2);
	}
	while (length($string) < $space) {
		$string .= $sep;
	}
	return $string;
}


### Input filters

# Reads a fasta file, returns two array pointers for header an sequence
# Access e.g. via $results[x]->[y]
#my %hash = aln2hash($file);
sub aln2hash {
	my $file = shift;
	my $cut = shift;
	if (!defined($cut)) {$cut = 0;}
	my %hash;
	my $length = -1;
	my $line = 0;
	open(FILE,"<$file") || die("aln2hash(): Could not open file: $file\n");
	while(<FILE>) {
		chomp;
		my ($species,$seq) = split(/\s+/,$_);
		if (!defined($species) || $line++ == 0) {next;}
		if ($length == -1) {$length = rindex($_," ");}				# detect line lenght
		$seq = substr($_,$length+1);

		if ($_ =~ /^ /) {
			next;
		}
		if (!defined($hash{$species})) {$hash{$species} = "";}
		$hash{$species} .= $seq;
	}
	close(FILE);
	return %hash;
}

# Reads a fasta file, returns two array pointers for header an sequence
# Access e.g. via $results[x]->[y]
#my %hash = cw2hash($file);
sub cw2hash {
	my $file = shift;
	my $cut = shift;
	if (!defined($cut)) {$cut = 0;}
	my %hash;
	my $length = -1;
	open(FILE,"<$file") || die("cw2hash(): Could not open file: $file\n");
	while(<FILE>) {
		chomp;
		my ($species,$seq) = split(/\s+/,$_);
		if (!defined($species) || $species eq "CLUSTAL" || $species eq "DIALIGN" || !defined($seq) || $seq !~ /[A-Za-z\-]/) {next;}
		if ($length == -1) {$length = rindex($_," ");}				# detect line lenght
		$seq = substr($_,$length+1);
		if (!defined($hash{$species})) {$hash{$species} = "";}
		$hash{$species} .= $seq;
	}
	close(FILE);
	return %hash;
}

# Reads a fasta file, returns two array pointers for header an sequence
# Access e.g. via $results[x]->[y]
#my %hash = stk2hash($file);
sub stk2hash {
	my $file = shift;
	my $cut = shift;
	if (!defined($cut)) {$cut = 0;}
	my %hash;
	my $length = -1;
	open(FILE,"<$file") || die("cw2hash(): Could not open file: $file\n");
	while(<FILE>) {
		chomp;
		if ($_ =~ /#/) {next;}
		my ($species,$seq) = split(/\s+/,$_);
		if (!defined($species) || $species eq "CLUSTAL" || $species eq "DIALIGN" || !defined($seq) || $seq !~ /[A-Za-z\-]/) {next;}
		if ($length == -1) {$length = rindex($_," ");}				# detect line lenght
		$seq = substr($_,$length+1);
		if (!defined($hash{$species})) {$hash{$species} = "";}
		$hash{$species} .= $seq;
	}
	close(FILE);
	return %hash;
}


# Reads a fasta file, returns two array pointers for header an sequence
# Access e.g. via $results[x]->[y]
#my %hash = fasta2hash($file);
sub fasta2hash {
	my $file = shift;
	my $cut = shift;
	if (!defined($cut)) {$cut = 0;}
	my %hash;
	open(FILE,"<$file") || die("fasta2hash(): Could not open file: $file\n");
	my $last_hash = "";
	while(<FILE>) {
		$_ =~ s/\r/\n/g;
		chomp;
		if ($_ =~ /^>/) {
			$_ =~ s/^>//;
			if ($cut) {
				$_ =~ s/ .*$//s;
			}
			$hash{$_} = "";
			$last_hash = $_;
			next;
		}
		$hash{$last_hash} .= $_;
	}
	close(FILE);
	return %hash;
}


